==> shelf_bind-0.2.0.tar.gz: lib/shelf_bind.dart <==
// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf.bind;

import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart';
import 'dart:mirrors';
import 'src/preconditions.dart';
import 'dart:async';
import 'dart:convert';

Handler bind(Type type, Map<String, Symbol> bindings, Function handler,
             {Symbol constructor: #build}) {
  return new _RequestParamBinding(type, constructor, bindings, handler).handler;
}

Handler bindJsonBody(Type type, Function handler) {
  return new _JsonBodyBinding(type, handler).handler;
}

class _JsonBodyBinding {
  final Type type;
  final Function boundHandler;

  Handler get handler => _handleRequest;

  _JsonBodyBinding(this.type, this.boundHandler) {
    ensure(type, isNotNull);
    ensure(boundHandler, isNotNull);
  }

  dynamic _handleRequest(Request request) {
    final jsonFuture = _extractJsonBody(request);
    return jsonFuture.then((json) {
      final classMirror = reflectClass(type);
      final instanceMirror =
          classMirror.newInstance(#fromJson, [json]);

      return boundHandler(instanceMirror.reflectee, request);
    });
  }

  Future<Map> _extractJsonBody(Request request) =>
      request.readAsString().then((s) => JSON.decode(s));

}


class _RequestParamBinding {
  final Type type;
  final Symbol constructorName;
  final Map<String, Symbol> memberBindings;
  final Function boundHandler;

  Handler get handler => _handleRequest;

  _RequestParamBinding(this.type, this.constructorName, this.memberBindings,
      this.boundHandler) {
    ensure(type, isNotNull);
    ensure(constructorName, isNotNull);
    ensure(memberBindings, isNot(isEmpty));
    ensure(boundHandler, isNotNull);
  }

  dynamic _handleRequest(Request request) {
    final namedArguments = _bindVariables(request);
    final classMirror = reflectClass(type);
    final instanceMirror =
        classMirror.newInstance(constructorName, [], namedArguments);

    return boundHandler(instanceMirror.reflectee, request);
  }

  Map<Symbol, dynamic> _bindVariables(Request request) {
    final pv = getPathVariables(request);
    final namedArguments = <Symbol, dynamic> {};
    memberBindings.forEach((k, v) {
      final value = pv[k];
      if (value != null) {
        namedArguments[v] = value;
      }
    });

    return namedArguments;
  }
}

==> shelf_bind-0.2.1.tar.gz: lib/shelf_bind.dart <==
// Copyright (c) 2014, The Shelf Bind project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library shelf.bind;

import 'package:shelf/shelf.dart';
import 'package:shelf_route/shelf_route.dart';
import 'dart:mirrors';
import 'src/preconditions.dart';
import 'dart:async';
import 'dart:convert';

class BindMode {
  final String literal;

  const BindMode._internal(this.literal);

  static const BindMode CONSTRUCTOR = const BindMode._internal("CONSTRUCTOR");
  static const BindMode PROPERTY = const BindMode._internal("PROPERTY");
//  static const BindMode CONSTRUCTOR_THEN_PROPERTY = const BindMode();
}

Handler bind(Type type, Map<String, Symbol> bindings, Function handler,
             { BindMode bindMode: BindMode.CONSTRUCTOR,
               Symbol constructor: #build }) {
  return new _RequestParamBinding(type, constructor, bindings, handler,
      bindMode).handler;
}

Handler bindJsonBody(Type type, Function handler) {
  return new _JsonBodyBinding(type, handler).handler;
}

class _JsonBodyBinding {
  final Type type;
  final Function boundHandler;

  Handler get handler => _handleRequest;

  _JsonBodyBinding(this.type, this.boundHandler) {
    ensure(type, isNotNull);
    ensure(boundHandler, isNotNull);
  }

  dynamic _handleRequest(Request request) {
    final jsonFuture = _extractJsonBody(request);
    return jsonFuture.then((json) {
      final classMirror = reflectClass(type);
      final instanceMirror =
          classMirror.newInstance(#fromJson, [json]);

      return boundHandler(instanceMirror.reflectee, request);
    });
  }

  Future<Map> _extractJsonBody(Request request) =>
      request.readAsString().then((s) => JSON.decode(s));

}


class _RequestParamBinding {
  final Type type;
  final Symbol constructorName;
  final Map<String, Symbol> memberBindings;
  final Function boundHandler;
  final BindMode bindMode;

  Handler get handler => _handleRequest;

  _RequestParamBinding(this.type, this.constructorName, this.memberBindings,
      this.boundHandler, this.bindMode) {
    ensure(type, isNotNull);
    ensure(constructorName, isNotNull);
    ensure(memberBindings, isNot(isEmpty));
    ensure(boundHandler, isNotNull);
    ensure(bindMode, isNotNull);
  }

  dynamic _handleRequest(Request request) {
    var instanceMirror = _createInstance(request);

    return boundHandler(instanceMirror.reflectee, request);
  }

  InstanceMirror _createInstance(Request request) {
    switch (bindMode) {
      case BindMode.CONSTRUCTOR:
        return _createInstanceViaConstructor(request);
      case BindMode.PROPERTY:
        return _createInstanceViaProperty(request);

      default:
        throw new StateError('unsupported bind mode: $bindMode');
    }
  }

  InstanceMirror _createInstanceViaConstructor(Request request) {
    final namedArguments = _bindVariables(request);
    final classMirror = reflectClass(type);
    final instanceMirror =
        classMirror.newInstance(constructorName, [], namedArguments);
    return instanceMirror;
  }

  InstanceMirror _createInstanceViaProperty(Request request) {
    final properties = _bindVariables(request);
    final classMirror = reflectClass(type);
    final instanceMirror =
        classMirror.newInstance(const Symbol(''), []);

    properties.forEach((property, value) {
      instanceMirror.setField(property, value);
    });
    return instanceMirror;
  }


  Map<Symbol, dynamic> _bindVariables(Request request) {
    final pv = getPathVariables(request);
    final namedArguments = <Symbol, dynamic> {};
    memberBindings.forEach((k, v) {
      final value = pv[k];
      if (value != null) {
        namedArguments[v] = value;
      }
    });

    return namedArguments;
  }
}
